#ifndef INPUT_H
#define INPUT_H

#include "toolbox.h"

class Input
{
    public:
        enum Actions
        {
            ACTION_LEFT = 0,
            ACTION_RIGHT = 1,
            ACTION_DOWN = 2,
            ACTION_UP = 3,

            ACTION_ATTACK = 4,
            ACTION_RUN = 5,

            NUM_ACTIONS
        };
        struct JoystickEvent
        {
            unsigned button;
            bool usingAxis;
            sf::Joystick::Axis axis;
            bool axisPositive;
            JoystickEvent():
                button(0),
                usingAxis(false),
                axis(sf::Joystick::X),
                axisPositive(false)
                {
                }
        };
        static const int BIND_TIME = 30;

        string getBindName(Actions action);
        bool isHeld(Actions action);
        bool isPressed(Actions action);
        bool isReleased(Actions action);

        vector<sf::Keyboard::Key> keyBindings;
        vector<JoystickEvent> joystickBindings;

        vector<string> actionNames;

        vector<bool> binded;
        vector<bool> keys;
        vector<bool> prevKeys;

        bool controllerChosen;
        bool binding;

        int bindTimer;
        int currentlyBinding;

        sf::Keyboard::Key currKey;

        unsigned currButton;
        sf::Joystick::Axis currAxis;
        bool currAxisPositive;
        bool bindingAxis;

        bool justSelected;

        vector<sf::Joystick::Axis> validAxes;

        bool joystickDetected;
        int controllerChooseTimer;
        bool usingJoystick;
        unsigned joystickId; // 0 - 7
        unsigned maxJoystickButtons; // Maximum number of buttons joystick supports

        void step();
        void reset();
        void rebindAction(Actions action);
        void loadDefaults();

        void save(estream& out);
        void load(estream& in);
        Input();
        ~Input();
    private:
        string getKeyName(const sf::Keyboard::Key key);
};

#endif // INPUT_H
