#ifndef ROOM_H
#define ROOM_H

#include "toolbox.h"
#include "Tile.h"
#include "Sprite.h"
#include "Trap.h"
#include "Item.h"
#include "BossDoor.h"
#include "Stairs.h" //you've been warned
#include "Particle.h"

class Entity;
class Player;
class Boss;

class Room
{
    public:
        enum LinkDirection
        {
            LINK_LEFT,
            LINK_UP,
            LINK_RIGHT,
            LINK_DOWN
        };
        static const int WIDTH;
        static const int HEIGHT;
        static const int TILE_SIZE;
        static const int SPAWN_RATES[12]; //ugh magic

        Room(int x, int y, int depth = 0, Room* parent = NULL);
        ~Room();

        void addEntity(Entity* other);
        void addParticle(Particle* p);
        void removeEntity(Entity* other);
        void removeEntities();
        Player* getPlayer();
        void step();
        void draw(sf::RenderTarget& window, bool debug);
        bool isSolid(int x, int y);
        Tile** getTiles();
        Tile& getTile(int x, int y);
        void randomize();
        void generateCollisionFlags();
        void generateDisplayIndices();
        void generateTileEntities();
        void link(Room* room, LinkDirection dir);
        bool linkExists(LinkDirection dir);
        void placeBossKey();
        void makeBossRoom();
        bool findCreeps();
        int x;
        int y;
        Room* parent;
        int depth;
        int w;
        int h;

        bool explored;
        int treasures;
        bool hasBoss;
    private:
        Sprite tilesetSprite;

        vector<Room*> links;

        Tile** tiles;
        vector<Entity*> entities;
        vector<Particle*> particles;
        Player* player;
        Boss* boss;
        bool stairsSpawned;
};

#endif // ROOM_H
