#include "CombatParticle.h"
#include "Player.h"
#include "Room.h"
const float CombatParticle::W = 10;
const float CombatParticle::H = 10;
const float CombatParticle::ARROW_SPEED = 3;
const float CombatParticle::MAGIC_BALL_SPEED = 2;
const float CombatParticle::DEATH_BALL_SPEED = 2.5;
const unsigned CombatParticle::MAGIC_BALL_LIFE = 200;
const unsigned CombatParticle::DEATH_BALL_LIFE = 400;
CombatParticle::CombatParticle(float x, float y, CombatParticleType pType, sf::Vector2f dir, int dmg):
    Entity(COMBAT_PARTICLE, x, y, W, H),
    pType(pType),
    dirVec(dir),
    sprite(nullptr),
    damage(dmg),
    reflected(false)
{
    float dirMag = sqrt(pow(dir.x,2)+pow(dir.y,2));
    if(dirMag > 0)
    {
        dirVec /= dirMag;
    }
    switch(pType)
    {
    case COMBAT_SLASH:
        w = 20;
        h = 20;
        if(COMBAT_UP || COMBAT_DOWN)
        {
            w = 16;
        }
        else
        {
            h = 16;
        }
        sprite = new Sprite(ResourceManager::getTexture("slash_effect.png"), 16, 16, -1, 1, -2, -2, false);

        break;
    case COMBAT_ARROW:
        sprite = new Sprite(ResourceManager::getTexture("projectiles.png"), 16, 16, -1, 0, 0, 0, false);
        sprite->setFrame(1);
        switch(findDir(dirVec))
        {
        case COMBAT_UP:
            dy = -ARROW_SPEED;
            sprite->setRotation(90);
            break;
        case COMBAT_DOWN:
            dy = ARROW_SPEED;
            sprite->setRotation(90);
            break;
        case COMBAT_LEFT:
            dx = -ARROW_SPEED;
            break;
        case COMBAT_RIGHT:
            dx = ARROW_SPEED;
            break;
        }
        break;
    case COMBAT_MAGIC_BALL:
        dx = MAGIC_BALL_SPEED*dirVec.x;
        dy = MAGIC_BALL_SPEED*dirVec.y;
        life = MAGIC_BALL_LIFE;
        sprite = new Sprite(ResourceManager::getTexture("projectiles.png"), 16, 16, -1, 0, 0, 0, false);
        sprite->setFrame(0);
        break;
    case COMBAT_DEATH_BALL:
        dx = DEATH_BALL_SPEED*dirVec.x;
        dy = DEATH_BALL_SPEED*dirVec.y;
        life = DEATH_BALL_LIFE;
        bouncy = true;
        sprite = new Sprite(ResourceManager::getTexture("projectiles.png"), 16, 16, -1, 0, 0, 0, false);
        sprite->setFrame(0);
        sprite->setColor(sf::Color::Red);
        break;
    }
}

CombatParticle::~CombatParticle()
{
    if(sprite)
    {
        delete sprite;
    }
}

void CombatParticle::step()
{
    if(life) life--;
    if(sprite && pType == COMBAT_SLASH)
    {
        if(sprite->isFinished())
        {
            die = true;
        }
        sprite->step();
    }
    if(pType == COMBAT_ARROW)
    {
        collideRoom();
        if(landedDown || landedLeft || landedRight || landedUp)
        {
            die = true;
        }
    }
    if(pType == COMBAT_MAGIC_BALL || pType == COMBAT_DEATH_BALL)
    {
        if(!life)
            die = true;
    }
    if(pType == COMBAT_DEATH_BALL)
    {
        collideRoom();
        if(landedDown || landedLeft || landedRight || landedUp)
        {
            ResourceManager::playSound("hit.wav");
        }
    }
    x += dx;
    y += dy;
}

void CombatParticle::draw(sf::RenderTarget& window)
{
    if(sprite)
    {
        sf::Vector2f offset;
        if(pType == COMBAT_SLASH)
        {
        switch(findDir(dirVec))
        {
        case COMBAT_UP:
            sprite->setRotation(0);
            offset.y = -4;
            break;
        case COMBAT_DOWN:
            sprite->setRotation(180);
            break;
        case COMBAT_LEFT:
            sprite->setRotation(270);
            offset.x = -4;
            break;
        case COMBAT_RIGHT:
            sprite->setRotation(90);
            break;
        }
        }
        sprite->setPosition(x + w/2 + offset.x,
                            y + h/2 + offset.y);

        sprite->draw(window);
    }
    else
    {
        sf::RectangleShape rect;
        rect.setSize(sf::Vector2f(w,h));
        rect.setPosition(x,y);
        if(pType == COMBAT_DEATH_BALL)
            rect.setFillColor(sf::Color::Red);
        window.draw(rect);
    }
}

void CombatParticle::doCollide(Entity* other)
{
    if(!collidesWith(other))
    {
        return;
    }
    switch(pType)
    {
    case COMBAT_SLASH:
        if(other->entityType == CREEP)
        {
            if(collidesWith(other))
            {
                if(other->hurt(damage))
                {
                    other->dx = 0;
                    other->dy = 0;
                }
            }
        }
        if(other->entityType == BOSS)
        {
            if(collidesWith(other))
            {
                Player* player = room->getPlayer();
                sf::Vector2f reflector((player->x+player->w/2) - (other->x+other->w/2),
                                       (player->y+player->h/2) - (other->y+other->h/2));
                float reflectorMag = sqrt(pow(reflector.x,2)+pow(reflector.y,2));
                if(reflectorMag > 0) reflector /= reflectorMag;
                reflector *= Player::MOVE_SPEED;
                player->dx += reflector.x;
                player->dy += reflector.y;
                player->attacking = false;
                //die = true;
            }
        }
        break;
    case COMBAT_ARROW:
        if((other->entityType == PLAYER ||
           other->entityType == CREEP) &&
           collidesWith(other))
        {
            if(other->hurt(damage))
            {
                die = true;
                other->dx += dx*0.7;
                other->dy += dy*0.7;
            }
        }
        break;
    case COMBAT_MAGIC_BALL:
        if(other->entityType == COMBAT_PARTICLE)
        {
            CombatParticle* cpOther = static_cast<CombatParticle*>(other);
            if(cpOther->pType == COMBAT_SLASH && !reflected)
            {
                dx *= -1;
                dy *= -1;
                reflected = true;
                ResourceManager::playSound("magic_reflect.wav");
            }
        }
        if(other->entityType == PLAYER && !reflected)
        {
            if(other->hurt(damage))
            {
                die = true;
                other->dx += dx*0.7;
                other->dy += dy*0.7;
                ResourceManager::playSound("magic_boom.wav");
            }
        }
        if(other->entityType == BOSS && reflected)
        {
            if(other->hurt(damage))
            {
                die = true;
                other->dx += dx*0.7;
                other->dy += dy*0.7;
                ResourceManager::playSound("magic_boom.wav");
            }
        }
        break;
    case COMBAT_DEATH_BALL:
        if(other->entityType == PLAYER && !reflected)
        {
            if(other->hurt(damage))
            {
                die = true;
                other->dx += dx*0.7;
                other->dy += dy*0.7;
                ResourceManager::playSound("magic_boom.wav");
            }
        }
        if(other->entityType == BOSS && reflected)
        {
            if(other->hurt(damage))
            {
                die = true;
                other->dx += dx*0.7;
                other->dy += dy*0.7;
                ResourceManager::playSound("magic_boom.wav");
            }
        }
        break;
    }
}

CombatParticle::CombatParticleDir CombatParticle::findDir(sf::Vector2f& vec)
{
    if(fabs(vec.x) > fabs(vec.y))
    {
        if(vec.x < 0)
        {
            return COMBAT_LEFT;
        }
        else
        {
            return COMBAT_RIGHT;
        }
    }
    else
    {
        if(vec.y < 0)
        {
            return COMBAT_UP;
        }
        else
        {
            return COMBAT_DOWN;
        }
    }
}
