#include "Input.h"

Input::Input()
{
    actionNames.resize(NUM_ACTIONS, "UNDEFINED");

    actionNames[ACTION_LEFT] = "Left";
    actionNames[ACTION_RIGHT] = "Right";
    actionNames[ACTION_UP] = "Up";
    actionNames[ACTION_DOWN] = "Down";

    actionNames[ACTION_ATTACK] = "Attack";
    actionNames[ACTION_RUN] = "Run";

    reset();
}

Input::~Input()
{
    //dtor
}

string Input::getBindName(Actions action)
{
    stringstream ss;
    if(!controllerChosen)
    {
    }
    else if(!binded[action])
    {
        if((Actions)currentlyBinding == action)
        {
            if(bindTimer == 0)
            {
                ss << "HOLD";
            }
            else
            {
                for(int i = 0; i < bindTimer/2; i++)
                {
                    ss << ".";
                }
            }
        }
    }
    else if(usingJoystick)
    {
        if(joystickBindings[action].usingAxis)
        {
            if(joystickBindings[action].axisPositive)
            {
                ss << "+ ";
            }
            else
            {
                ss << "- ";
            }
            ss << "Axis " << joystickBindings[action].axis;
        }
        else
        {
            ss << "Button " << joystickBindings[action].button;
        }
    }
    else
    {
        ss << getKeyName(keyBindings[action]);
    }
    return ss.str();
}

bool Input::isHeld(Actions action)
{
    return keys[action];
}

bool Input::isPressed(Actions action)
{
    return keys[action] && !prevKeys[action];
}

bool Input::isReleased(Actions action)
{
    return !keys[action] && prevKeys[action];
}

void Input::step()
{
    if(!controllerChosen)
    {
        bool noButtonPressed = true;
        for(unsigned i = 0; i < 8; i++)
        {
            if(sf::Joystick::isConnected(i))
            {
                joystickDetected = true;
                unsigned numButtons = sf::Joystick::getButtonCount(i);
                maxJoystickButtons = numButtons;
                for(unsigned buttonId = 0; buttonId < numButtons; buttonId++)
                {
                    if(sf::Joystick::isButtonPressed(i, buttonId))
                    {
                        noButtonPressed = false;
                        if(usingJoystick == false ||
                           joystickId != i)
                        {
                            controllerChooseTimer = 0;
                        }
                        joystickId = i;
                        usingJoystick = true;
                        controllerChooseTimer++;
                    }
                }
            }
        }
        for(unsigned i = 0; i < sf::Keyboard::KeyCount; i++)
        {
            if(sf::Keyboard::isKeyPressed((sf::Keyboard::Key)i))
            {
                noButtonPressed = false;
                if(usingJoystick == true)
                {
                    controllerChooseTimer = 0;
                }
                usingJoystick = false;
                controllerChooseTimer++;
            }
        }
        if(noButtonPressed)
        {
            controllerChooseTimer = 0;
        }
        if(controllerChooseTimer >= BIND_TIME)
        {
            controllerChooseTimer = BIND_TIME;
            controllerChosen = true;
            if(usingJoystick)
            {
                validAxes.clear();
                validAxes.push_back(sf::Joystick::X);
                validAxes.push_back(sf::Joystick::Y);
                validAxes.push_back(sf::Joystick::Z);
                validAxes.push_back(sf::Joystick::R);
                validAxes.push_back(sf::Joystick::U);
                validAxes.push_back(sf::Joystick::V);
                validAxes.push_back(sf::Joystick::PovX);
                validAxes.push_back(sf::Joystick::PovY);

                /// check valid axes
                for(unsigned i = 0; i < validAxes.size(); i++)
                {
                    if(!sf::Joystick::hasAxis(joystickId, validAxes[i]))
                    {
                        validAxes.erase(validAxes.begin() + i);
                        i--;
                    }
                }
            }
            justSelected = true;
        }
    }
    else if(binding)
    {
        bool allBinded = true;
        unsigned action;
        for(action = 0; action < binded.size(); action++)
        {
            if(!binded[action])
            {
                allBinded = false;
                currentlyBinding = action;
                //printf("Trying to bind %d\n", action);
                break;
            }
        }

        if(allBinded)
        {
            binding = false;
        }
        else
        {
            if(!usingJoystick)
            {
                int keysHeldCount = 0;
                sf::Keyboard::Key prevKey = currKey;
                for(unsigned i = 0; i < sf::Keyboard::KeyCount; i++)
                {
                    if(sf::Keyboard::isKeyPressed((sf::Keyboard::Key)i))
                    {
                        keysHeldCount++;
                        currKey = (sf::Keyboard::Key)i;
                    }
                }
                if(keysHeldCount == 0 && justSelected)
                {
                    justSelected = false;
                }
                //printf("heldCount: %d, currKey: %d ?== prevKey: %d\n", keysHeldCount, (int)(currKey), (int)(prevKey));
                if(!justSelected && keysHeldCount == 1 && currKey == prevKey)
                {
                    if(bindTimer < BIND_TIME)
                    {
                        bindTimer++;
                        if(bindTimer == BIND_TIME)
                        {
                            bindTimer = 0;
                            keyBindings[action] = currKey;
                            binded[action] = true;
                            justSelected = true;
                            //printf("Action %d binded to key %d!!!\n", action, currKey);
                        }
                    }
                }
                else
                {
                    bindTimer = 0;
                }
            }
            else
            {
                unsigned prevButton = currButton;

                int keysHeldCount = 0;
                for(unsigned i = 0; i < maxJoystickButtons; i++)
                {
                    if(sf::Joystick::isButtonPressed(joystickId, i))
                    {
                        keysHeldCount++;
                        currButton = i;
                        bindingAxis = false;
                    }
                }

                sf::Joystick::Axis lastAxis = currAxis;

                for(unsigned i = 0; i < validAxes.size(); i++)
                {
                    float position = sf::Joystick::getAxisPosition(joystickId, validAxes[i]);
                    if(fabs(position) > 50)
                    {
                        bindingAxis = true;
                        keysHeldCount++;
                        currAxis = validAxes[i];
                        currAxisPositive = position > 0;
                    }
                }

                if(keysHeldCount == 0 && justSelected)
                {
                    justSelected = false;
                }
                if(!justSelected && keysHeldCount == 1 &&
                   ((!bindingAxis && currButton == prevButton) || (bindingAxis && currAxis == lastAxis)))
                {
                    if(bindTimer < BIND_TIME)
                    {
                        bindTimer++;
                        if(bindTimer == BIND_TIME)
                        {
                            bindTimer = 0;
                            joystickBindings[action].usingAxis = bindingAxis;
                            joystickBindings[action].button = currButton;
                            joystickBindings[action].axis = currAxis;
                            joystickBindings[action].axisPositive = currAxisPositive;
                            binded[action] = true;
                            printf("Bound %s!\n", actionNames[action].c_str());
                            justSelected = true;
                        }
                    }
                }
                else
                {
                    bindTimer = 0;
                }
            }
        }
    }
    else
    {
        if(!usingJoystick)
        {
            for(unsigned i = 0; i < prevKeys.size(); i++)
            {
                prevKeys[i] = keys[i];
                keys[i] = sf::Keyboard::isKeyPressed(keyBindings[i]);
            }
        }
        else
        {
            for(unsigned i = 0; i < prevKeys.size(); i++)
            {
                prevKeys[i] = keys[i];
                if(!joystickBindings[i].usingAxis)
                {
                    keys[i] = sf::Joystick::isButtonPressed(joystickId, joystickBindings[i].button);
                }
                else
                {
                    float pos = sf::Joystick::getAxisPosition(joystickId, joystickBindings[i].axis);
                    if(joystickBindings[i].axisPositive)
                    {
                        keys[i] = pos > 50;
                    }
                    else
                    {
                        keys[i] = pos < -50;
                    }
                }
            }
        }
    }
}

void Input::reset()
{
    currentlyBinding = 0;
    joystickDetected = false;
    keyBindings.resize(NUM_ACTIONS, sf::Keyboard::Space);
    joystickBindings.resize(NUM_ACTIONS, JoystickEvent());
    binded.resize(NUM_ACTIONS, false);
    keys.resize(NUM_ACTIONS, false);
    prevKeys.resize(NUM_ACTIONS, false);
    for(unsigned i = 0; i < NUM_ACTIONS; i++)
    {
        binded[i] = false;
    }

    bindTimer = 0;
    binding = true;
    usingJoystick = false;
    controllerChosen = false;
    controllerChooseTimer = 0;
}

void Input::rebindAction(Actions action)
{
    if(binding)
    {
        printf("Cannot rebind action, currently binding something else!\n");
        return;
    }
    currentlyBinding = action;
    binded[action] = false;
    binding = true;
}

void Input::loadDefaults()
{
    for(unsigned i = 0; i < NUM_ACTIONS; i++)
    {
        binded[i] = true;
    }
    binding = false;
    usingJoystick = false;
    controllerChosen = true;
    keyBindings[ACTION_LEFT] = sf::Keyboard::Left;
    keyBindings[ACTION_RIGHT] = sf::Keyboard::Right;
    keyBindings[ACTION_DOWN] = sf::Keyboard::Down;
    keyBindings[ACTION_UP] = sf::Keyboard::Up;

    keyBindings[ACTION_ATTACK] = sf::Keyboard::X;
    keyBindings[ACTION_RUN] = sf::Keyboard::Z;
}

void Input::save(estream& out)
{
    out << usingJoystick;
    if(usingJoystick)
    {
        out << joystickId;
    }
    for(unsigned i = 0; i < actionNames.size(); i++)
    {
        if(usingJoystick)
        {
            out << joystickBindings[i].usingAxis;
            out << joystickBindings[i].axisPositive;
            out << (unsigned)(joystickBindings[i].axis);
            out << joystickBindings[i].button;
        }
        else
        {
            out << (unsigned)(keyBindings[i]);
        }
    }
}
void Input::load(estream& in)
{
    loadDefaults();
    in >> usingJoystick;
    bool validControllerConnected = true;
    if(usingJoystick)
    {
        in >> joystickId;
        if(!sf::Joystick::isConnected(joystickId))
        {
            validControllerConnected = false;
            for(unsigned i = 0; i < 8; i++)
            {
                if(sf::Joystick::isConnected(i))
                {
                    joystickId = i;
                    validControllerConnected = true;
                    break;
                }
            }
        }
    }

    for(unsigned i = 0; i < actionNames.size(); i++)
    {
        if(usingJoystick)
        {
            in >> joystickBindings[i].usingAxis;
            in >> joystickBindings[i].axisPositive;
            unsigned axisIndex;
            in >> axisIndex;
            joystickBindings[i].axis = static_cast<sf::Joystick::Axis>(axisIndex);
            in >> joystickBindings[i].button;
            if(!sf::Joystick::hasAxis(joystickId, joystickBindings[i].axis))
            {
                validControllerConnected = false;
            }
            if(joystickBindings[i].button >= sf::Joystick::getButtonCount(joystickId))
            {
                validControllerConnected = false;
            }
        }
        else
        {
            unsigned keyIndex;
            in >> keyIndex;
            keyBindings[i] = static_cast<sf::Keyboard::Key>(keyIndex);
        }
    }

    if(usingJoystick)
    {
        if(!validControllerConnected)
        {
            loadDefaults();
        }
        else
        {
            maxJoystickButtons = sf::Joystick::getButtonCount(joystickId);
        }
    }
}


string Input::getKeyName( const sf::Keyboard::Key key ) {
    switch( key ) {
    default:
    case sf::Keyboard::Unknown:
        return "Unknown";
    case sf::Keyboard::A:
        return "A";
    case sf::Keyboard::B:
        return "B";
    case sf::Keyboard::C:
        return "C";
    case sf::Keyboard::D:
        return "D";
    case sf::Keyboard::E:
        return "E";
    case sf::Keyboard::F:
        return "F";
    case sf::Keyboard::G:
        return "G";
    case sf::Keyboard::H:
        return "H";
    case sf::Keyboard::I:
        return "I";
    case sf::Keyboard::J:
        return "J";
    case sf::Keyboard::K:
        return "K";
    case sf::Keyboard::L:
        return "L";
    case sf::Keyboard::M:
        return "M";
    case sf::Keyboard::N:
        return "N";
    case sf::Keyboard::O:
        return "O";
    case sf::Keyboard::P:
        return "P";
    case sf::Keyboard::Q:
        return "Q";
    case sf::Keyboard::R:
        return "R";
    case sf::Keyboard::S:
        return "S";
    case sf::Keyboard::T:
        return "T";
    case sf::Keyboard::U:
        return "U";
    case sf::Keyboard::V:
        return "V";
    case sf::Keyboard::W:
        return "W";
    case sf::Keyboard::X:
        return "X";
    case sf::Keyboard::Y:
        return "Y";
    case sf::Keyboard::Z:
        return "Z";
    case sf::Keyboard::Num0:
        return "Num0";
    case sf::Keyboard::Num1:
        return "Num1";
    case sf::Keyboard::Num2:
        return "Num2";
    case sf::Keyboard::Num3:
        return "Num3";
    case sf::Keyboard::Num4:
        return "Num4";
    case sf::Keyboard::Num5:
        return "Num5";
    case sf::Keyboard::Num6:
        return "Num6";
    case sf::Keyboard::Num7:
        return "Num7";
    case sf::Keyboard::Num8:
        return "Num8";
    case sf::Keyboard::Num9:
        return "Num9";
    case sf::Keyboard::Escape:
        return "Escape";
    case sf::Keyboard::LControl:
        return "LControl";
    case sf::Keyboard::LShift:
        return "LShift";
    case sf::Keyboard::LAlt:
        return "LAlt";
    case sf::Keyboard::LSystem:
        return "LSystem";
    case sf::Keyboard::RControl:
        return "RControl";
    case sf::Keyboard::RShift:
        return "RShift";
    case sf::Keyboard::RAlt:
        return "RAlt";
    case sf::Keyboard::RSystem:
        return "RSystem";
    case sf::Keyboard::Menu:
        return "Menu";
    case sf::Keyboard::LBracket:
        return "LBracket";
    case sf::Keyboard::RBracket:
        return "RBracket";
    case sf::Keyboard::SemiColon:
        return "SemiColon";
    case sf::Keyboard::Comma:
        return "Comma";
    case sf::Keyboard::Period:
        return "Period";
    case sf::Keyboard::Quote:
        return "Quote";
    case sf::Keyboard::Slash:
        return "Slash";
    case sf::Keyboard::BackSlash:
        return "BackSlash";
    case sf::Keyboard::Tilde:
        return "Tilde";
    case sf::Keyboard::Equal:
        return "Equal";
    case sf::Keyboard::Dash:
        return "Dash";
    case sf::Keyboard::Space:
        return "Space";
    case sf::Keyboard::Return:
        return "Return";
    case sf::Keyboard::BackSpace:
        return "BackSpace";
    case sf::Keyboard::Tab:
        return "Tab";
    case sf::Keyboard::PageUp:
        return "PageUp";
    case sf::Keyboard::PageDown:
        return "PageDown";
    case sf::Keyboard::End:
        return "End";
    case sf::Keyboard::Home:
        return "Home";
    case sf::Keyboard::Insert:
        return "Insert";
    case sf::Keyboard::Delete:
        return "Delete";
    case sf::Keyboard::Add:
        return "Add";
    case sf::Keyboard::Subtract:
        return "Subtract";
    case sf::Keyboard::Multiply:
        return "Multiply";
    case sf::Keyboard::Divide:
        return "Divide";
    case sf::Keyboard::Left:
        return "Left";
    case sf::Keyboard::Right:
        return "Right";
    case sf::Keyboard::Up:
        return "Up";
    case sf::Keyboard::Down:
        return "Down";
    case sf::Keyboard::Numpad0:
        return "Numpad0";
    case sf::Keyboard::Numpad1:
        return "Numpad1";
    case sf::Keyboard::Numpad2:
        return "Numpad2";
    case sf::Keyboard::Numpad3:
        return "Numpad3";
    case sf::Keyboard::Numpad4:
        return "Numpad4";
    case sf::Keyboard::Numpad5:
        return "Numpad5";
    case sf::Keyboard::Numpad6:
        return "Numpad6";
    case sf::Keyboard::Numpad7:
        return "Numpad7";
    case sf::Keyboard::Numpad8:
        return "Numpad8";
    case sf::Keyboard::Numpad9:
        return "Numpad9";
    case sf::Keyboard::F1:
        return "F1";
    case sf::Keyboard::F2:
        return "F2";
    case sf::Keyboard::F3:
        return "F3";
    case sf::Keyboard::F4:
        return "F4";
    case sf::Keyboard::F5:
        return "F5";
    case sf::Keyboard::F6:
        return "F6";
    case sf::Keyboard::F7:
        return "F7";
    case sf::Keyboard::F8:
        return "F8";
    case sf::Keyboard::F9:
        return "F9";
    case sf::Keyboard::F10:
        return "F10";
    case sf::Keyboard::F11:
        return "F11";
    case sf::Keyboard::F12:
        return "F12";
    case sf::Keyboard::F13:
        return "F13";
    case sf::Keyboard::F14:
        return "F14";
    case sf::Keyboard::F15:
        return "F15";
    case sf::Keyboard::Pause:
        return "Pause";
    }
}
