#include "Particle.h"

Particle::Particle(ParticleType pType, float x, float y):
    pType(pType),
    pos(x, y),
    vel(0, 0),
    accel(0, 0),
    lifeTime(60),
    die(false)
{
    switch(pType)
    {
        case TYPE_BLOOD:
        {
            float speed = rand()/(float)RAND_MAX + 0.5;
            float angle = rand()/(float)RAND_MAX * 2 * PI;
            vel.x = cosf(angle) * speed;
            vel.y = sinf(angle) * speed;
            lifeTime = rand()%30 + 30;
            break;
        }
        case TYPE_WHITE_BLOOD:
        {
            float speed = rand()/(float)RAND_MAX + 0.5;
            float angle = rand()/(float)RAND_MAX * 2 * PI;
            vel.x = cosf(angle) * speed;
            vel.y = sinf(angle) * speed;
            lifeTime = rand()%30 + 30;
            break;
        }
        case TYPE_DUST:
            lifeTime = rand()%30 + 0;
            break;
        case TYPE_SHINY:
            accel.y = -0.01;
            lifeTime = 20 + rand()%30 + 0;
            break;
    }
    lifeTimer = lifeTime;
}

Particle::~Particle()
{
    //dtor
}

void Particle::draw(sf::RenderTarget& window)
{
    vel.x *= 0.99;
    vel.y *= 0.99;
    vel += accel;
    pos += vel;
    lifeTimer--;
    if(lifeTimer < 0)
    {
        die = true;
        lifeTimer = 0;
    }
    if(pType == TYPE_BLOOD)
    {
        sf::RectangleShape box({2, 2});
        box.setPosition(pos.x, pos.y);
        box.setFillColor(sf::Color(255, 0, 0, 255 * (lifeTimer / (float)lifeTime)));
        window.draw(box);
    }
    else if(pType == TYPE_WHITE_BLOOD)
    {
        sf::RectangleShape box({2, 2});
        box.setRotation(rand()%90);
        box.setPosition(pos.x, pos.y);
        box.setFillColor(sf::Color(255, 255, 255, 255 * (lifeTimer / (float)lifeTime)));
        window.draw(box);
    }
    else if(pType == TYPE_DUST)
    {
        sf::RectangleShape box({2, 2});
        box.setPosition(pos.x, pos.y);
        box.setFillColor(sf::Color(255, 255, 255, 200 * (lifeTimer / (float)lifeTime)));
        window.draw(box);
    }
    else if(pType == TYPE_SHINY)
    {
        sf::RectangleShape box({2, 2});
        box.setPosition(pos.x, pos.y);
        box.setRotation(rand()%90);
        box.setFillColor(sf::Color(255, 255, 255, 255 * (lifeTimer / (float)lifeTime)));
        window.draw(box);
    }
}
