#include "RoomDatabase.h"
#include "Room.h"

vector<Room*> RoomDatabase::rooms;
map<int, vector<RoomDatabase::RoomSetup> > RoomDatabase::roomSetups;
map<int, vector<int>> RoomDatabase::currSetupIndices;
int RoomDatabase::currDepth;

void RoomDatabase::initialize()
{
    for(int i = 0; i < 16; i++)
    {
        roomSetups.insert(pair<int, vector<RoomSetup> >(i, vector<RoomSetup>()));
        currSetupIndices.insert(pair<int, vector<int> >(i, vector<int>()));
    }
    currDepth = -1;
    for(int i = 0; i < 16; i++)
    {
        if(i == 1 || i == 3 || i == 5 || i == 7 || i == 15)
        {
            stringstream fileName("");
            fileName << "lvl/";

            fileName << i;
            ifstream dataFile(fileName.str());
            if(dataFile.is_open())
            {
                vector<vector<char>> buffer;
                buffer.resize(Room::HEIGHT, vector<char>());

                string line;
                int lineNum = 0;
                int fileLineNum = 0;
                int minDepth = 0;
                int maxDepth = 11;
                while(getline(dataFile, line))
                {
                    fileLineNum++;

                    //determine if this is a header (min/max depth) before proceeding
                    if (lineNum == 0)
                    {
                        istringstream iss(line);
                        vector<string> tokens(istream_iterator<string>{iss},
                            istream_iterator<string>());
                        if (tokens.size() == 2 && tokens[0].length() > 0 && tokens[1].length() > 0)
                        {
                            stringstream ss(tokens[0]);
                            ss >> minDepth;
                            ss << tokens[1];
                            ss >> maxDepth;
                            continue; //don't try to parse this as data obvs
                        }
                    }

                    bool validLine = false;
                    buffer[lineNum].clear();
                    for(unsigned c = 0; c < line.length(); c++)
                    {
                        char character = line[c];
                        if(character == '.' ||
                           (character >= '1' && character <= '6'))
                        {
                            buffer[lineNum].push_back(character);

                            validLine = true;
                        }
                    }

                    if(validLine)
                    {
                        lineNum = (lineNum+1) % Room::HEIGHT;
                        if(lineNum == 0)
                        {
                            for(int y = 0; y < Room::HEIGHT; y++)
                            {
                                if((int)buffer[y].size() != Room::WIDTH)
                                {
                                    printf("Error loading room of type %d, line %d is not correct length.\n", (int)i, (int)fileLineNum);
                                }
                            }
                            Room* newTemplate = new Room(0, 0);

                            Tile** tiles = newTemplate->getTiles();
                            for(int x = 0; x < Room::WIDTH; x++)
                            {
                                for(int y = 0; y < Room::HEIGHT; y++)
                                {
                                    TileType type;
                                    switch(buffer[y][x])
                                    {
                                    case '.':
                                        type = TILE_NONE;
                                        break;
                                    case '1':
                                        type = TILE_SOLID;
                                        break;
                                    case '2':
                                        type = TILE_MONSTER_SPAWN;
                                        break;
                                    case '3':
                                        type = TILE_ARROWTRAP;
                                        break;
                                    case '4':
                                        type = TILE_REWARD;
                                        break;
                                    case '5':
                                        type = TILE_SPIKE;
                                        break;
                                    case '6':
                                        type = TILE_HOLE;
                                        break;
                                    default:
                                        printf("unknown tile type! '%c'\n", buffer[y][x]);
                                        type = TILE_NONE;
                                        break;
                                    }
                                    tiles[x][y].type = type;

                                }
                            }
                            rooms.push_back(newTemplate);
                            int roomIndex = rooms.size()-1;
                            roomSetups[i].push_back(RoomSetup(roomIndex, 0, minDepth, maxDepth));
                            if(i == 1)
                            {
                                roomSetups[2].push_back(RoomSetup(roomIndex, 1, minDepth, maxDepth));
                                roomSetups[4].push_back(RoomSetup(roomIndex, 2, minDepth, maxDepth));
                                roomSetups[8].push_back(RoomSetup(roomIndex, 3, minDepth, maxDepth));
                            }
                            else if(i == 3)
                            {
                                roomSetups[6].push_back(RoomSetup(roomIndex, 1, minDepth, maxDepth));
                                roomSetups[12].push_back(RoomSetup(roomIndex, 2, minDepth, maxDepth));
                                roomSetups[9].push_back(RoomSetup(roomIndex, 3, minDepth, maxDepth));
                            }
                            else if(i == 5)
                            {
                                roomSetups[10].push_back(RoomSetup(roomIndex, 1, minDepth, maxDepth));
                                roomSetups[5].push_back(RoomSetup(roomIndex, 2, minDepth, maxDepth));
                                roomSetups[10].push_back(RoomSetup(roomIndex, 3, minDepth, maxDepth));
                            }
                            else if(i == 7)
                            {
                                roomSetups[14].push_back(RoomSetup(roomIndex, 1, minDepth, maxDepth));
                                roomSetups[13].push_back(RoomSetup(roomIndex, 2, minDepth, maxDepth));
                                roomSetups[11].push_back(RoomSetup(roomIndex, 3, minDepth, maxDepth));
                            }
                            else if(i == 15)
                            {
                                roomSetups[15].push_back(RoomSetup(roomIndex, 1, minDepth, maxDepth));
                                roomSetups[15].push_back(RoomSetup(roomIndex, 2, minDepth, maxDepth));
                                roomSetups[15].push_back(RoomSetup(roomIndex, 3, minDepth, maxDepth));
                            }
                            minDepth = 0;
                            maxDepth = 12; //reset these for the next room, as the header is optional
                        }
                    }
                    else
                    {
                        if(lineNum != 0)
                        {
                            printf("RoomDatabase::possible error detected loading room type %d!\n", i);
                        }
                    }
                }

                dataFile.close();
            }
        }
    }
}

void RoomDatabase::cleanUp()
{
    for(unsigned i = 0; i < rooms.size(); i++)
    {
        delete rooms[i];
    }

}

void RoomDatabase::setDepth(int depth)
{
    depth = min(depth,11);
    if (depth == currDepth) return;
    currDepth = depth;
    for (int i = 0; i < 16; i++)
    {
        currSetupIndices[i].clear();
        for (int n = 0; n < roomSetups[i].size(); n++)
        {
            RoomSetup setup = roomSetups[i][n];
            if (currDepth >= setup.minDepth && currDepth <= setup.maxDepth)
            {
                currSetupIndices[i].push_back(n);
            }
        }
    }
}

void RoomDatabase::getRandomRoomLayout(Room* room, int type)
{
    int depth = room->depth;
    setDepth(depth); //to generate rooms of the correct depth

    int numRooms = currSetupIndices[type].size();
    if (numRooms == 0)
    {
        printf("No legal rooms for type %d depth %d!\n",type,depth);
    }
    int setupIndex = currSetupIndices[type][rand()%numRooms];
    RoomSetup setup = roomSetups[type][setupIndex];

    Room* refRoom = rooms[setup.roomIndex];
    int rotation = setup.rotation;

    Tile** tiles = room->getTiles();
    Tile** refTiles = refRoom->getTiles();

    for(int x = 0; x < room->w; x++)
    {
        for(int y = 0; y < room->h; y++)
        {
            tiles[x][y].type = refTiles[x][y].type;
        }
    }

    //create buffer for rotate.
    Tile** buffer = new Tile*[room->w];
    for(int x = 0; x < room->w; x++)
    {
        buffer[x] = new Tile[room->h];
    }
    for(int i = 0; i < rotation; i++)
    {
        for(int x = 0; x < room->w; x++)
        {
            for(int y = 0; y < room->h; y++)
            {
                buffer[x][y].type = tiles[x][y].type;
            }
        }
        for(int x = 0; x < room->w; x++)
        {
            for(int y = 0; y < room->h; y++)
            {
                tiles[room->w - 1 - y][x].type = buffer[x][y].type;
            }
        }

    }
    //cleanup buffer.
    for(int x = 0; x < room->w; x++)
    {
        delete[] buffer[x];
    }
    delete[] buffer;
}
